# typicality SOME Likert-scale version
var ns, nm, nd, states[ns], psetM[127,8], R0[nm,ns], thresholds[nd-1],
pi, PPCaTMP[ns,nd], PPCbTMP[ns,nd], salH[8] , salL[8], salience[2,7];
#
model{
  # prior parameters
  piInv ~ dgamma(0.5,0.5) # precision of utility function
  pi = 1/piInv
  for (i in 1:7){ # salience of alternatives to "some"
    salH[i] ~ dbeta(1,1)
    salL[i] ~ dbeta(1,1)
  }
  for (i in 1:(nd-1)) { # thresholds for cumulative-normal link function
    thresholds[i] ~ dnorm(i/nd, (pow(2*nd,2)))
  }
  sigma ~ dunif(0,0.4) # standard deviation for cum-norm link function
  theta ~ dnorm(0.5,25) # threshold for logit link function; standard deviation 0.2
  gammaInv ~ dgamma(1,1) # gain for logit link function
  gamma = 1/gammaInv
  # auxiliary 
  for (s in 1:ns) {
    for (t in 1:ns) {
      U[s,t] = exp(- abs(states[s] - states[t])**2 *pi)
    }
  }
  for (i in 1:127){
    for (m in 1:nm){
      psetPtmpH[i,m] = ifelse(psetM[i,m] == 1, salH[m], 1-salH[m])
      psetPtmpL[i,m] = ifelse(psetM[i,m] == 1, salL[m], 1-salL[m])
    }
    psetPH[i] = prod(psetPtmpH[i,])
    psetPL[i] = prod(psetPtmpL[i,])
  }
  # get EUs
  for (i in 1:ns){    # states
    for (j in 1:nm){  # messages
      EUs[i,j] = sum(R0[j,] * U[i,])  #- cost[j]
    }
  }
  # model
  for(s in 1:127){
    for (i in 1:ns){    # states
      sEUsH[s,i] = psetPH[s] * ( EUs[i,8] - min(EUs[i,pindM[s,]]) ) / (max(EUs[i,pindM[s,]] - min(EUs[i,pindM[s,]])))
      sEUsL[s,i] = psetPL[s] * ( EUs[i,8] - min(EUs[i,pindM[s,]]) ) / (max(EUs[i,pindM[s,]] - min(EUs[i,pindM[s,]])))
    }
  }
  for (s in 1:ns){
    EUscaledH[s] = sum(sEUsH[,s]) # latent degree exps A & C
    EUscaledL[s] = sum(sEUsL[,s]) # latent degree exps B & D
  }
  # latent probability pa & pb & likelihood for data sets da & db
  for (s in 1:ns) {
    pa[s,1]  = phi( ( thresholds[1] - EUscaledH[s] ) / sigma )
    pa[s,nd] = 1 - phi( ( thresholds[nd-1] - EUscaledH[s] ) / sigma )
    pb[s,1]  = phi( ( thresholds[1] - EUscaledL[s] ) / sigma )
    pb[s,nd] = 1 - phi( ( thresholds[nd-1] - EUscaledL[s] ) / sigma )
    for (d in 2:(nd-1)){
      pa[s,d] = max( 0 ,  phi( ( thresholds[ d ] - EUscaledH[s] ) / sigma )
                     - phi( ( thresholds[d-1] - EUscaledH[s] ) / sigma ) ) 
      pb[s,d] = max( 0 ,  phi( ( thresholds[ d ] - EUscaledL[s] ) / sigma )
                     - phi( ( thresholds[d-1] - EUscaledL[s] ) / sigma ) ) 
    }
    da[s,] ~ dmulti(pa[s,],nVPa[s])
    PPCaTMP[s,] ~ dmulti(pa[s,],nVPa[s])
    PPCa[s] = sum(PPCaTMP[s,] * 1:7) / nVPa[s]
    db[s,] ~ dmulti(pb[s,],nVPb[s])
    PPCbTMP[s,] ~ dmulti(pb[s,],nVPb[s])
    PPCb[s] = sum(PPCbTMP[s,] * 1:7) / nVPb[s]
  }
  # latent probability pc & pd & likelihood for data sets dc & dd
  for (s in 1:ns) {
    pc[s] = 1 / (1+ exp(-gamma * (EUscaledH[s] - theta)))  
    dc[s,2] ~ dbinom(pc[s],nVPc[s])
    PPCcTMP[s] ~ dbinom(pc[s],nVPc[s])
    PPCc[s] = PPCcTMP[s] / nVPc[s]
    pd[s] = 1 / (1+ exp(-gamma * (EUscaledL[s] - theta)))  
    dd[s,2] ~ dbinom(pd[s],nVPd[s])
    PPCdTMP[s] ~ dbinom(pd[s],nVPd[s])
    PPCd[s] = PPCdTMP[s] / nVPd[s]
  }
  # for nicer output
  salience[1,] = salH[1:7]
  salience[2,] = salL[1:7]
}